import os

__all__ = list()
for fileName in os.listdir(os.path.dirname(__file__)):
    if fileName.endswith(".py") \
            and os.path.isfile(os.path.join(os.path.dirname(__file__), fileName)) \
            and not fileName.endswith('__init__.py'):

        __all__.append(os.path.basename(fileName[:-3]))

