"""
__init__ script for chaosmeter package
"""

from chaosmeter.ChaosMeter import chaosMeterVersion

__version__ = chaosMeterVersion
__author__ = "Ali Parsai"
__package__ = 'chaosmeter'
__license__ = 'GNU GPLv3'
__url__ = 'https://chaosmeter.parsai.net/'


