#!/usr/bin/env python3

"""
__main__ script for chaosmeter package
"""

from chaosmeter import ChaosMeter
import sys


def entryPoint():
    ChaosMeter.main()


if __name__ == "__main__":
    sys.exit(entryPoint())
